@@ -21,6 +21,7 @@ import numpy as np
 import tensorflow as tf
 import tensorflow_datasets as tds
 
+from ..configs.config import DecoderConfig
 from ..utils.utils import preprocess_paths
 
 ENGLISH_CHARACTERS = [" ", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m",
@@ -30,9 +31,7 @@ ENGLISH_CHARACTERS = [" ", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k"
 class TextFeaturizer(metaclass=abc.ABCMeta):
     def __init__(self, decoder_config: dict):
         self.scorer = None
-        self.decoder_config = decoder_config
-        if self.decoder_config.get("vocabulary", None) is not None:
-            self.decoder_config["vocabulary"] = preprocess_paths(self.decoder_config["vocabulary"])
+        self.decoder_config = DecoderConfig(decoder_config)
         self.blank = None
         self.tokens2indices = {}
         self.tokens = []
@@ -100,12 +99,12 @@ class CharFeaturizer(TextFeaturizer):
 
     def __init_vocabulary(self):
         lines = []
-        if self.decoder_config.get("vocabulary", None) is not None:
-            with codecs.open(self.decoder_config["vocabulary"], "r", "utf-8") as fin:
+        if self.decoder_config.vocabulary is not None:
+            with codecs.open(self.decoder_config.vocabulary, "r", "utf-8") as fin:
                 lines.extend(fin.readlines())
         else:
             lines = ENGLISH_CHARACTERS
-        self.blank = 0 if self.decoder_config.get("blank_at_zero", True) else None
+        self.blank = 0 if self.decoder_config.blank_at_zero else None
         self.tokens2indices = {}
         self.tokens = []
         index = 1 if self.blank == 0 else 0
@@ -120,10 +119,7 @@ class CharFeaturizer(TextFeaturizer):
         self.tokens.insert(self.blank, "")  # add blank token to tokens
         self.num_classes = len(self.tokens)
         self.tokens = tf.convert_to_tensor(self.tokens, dtype=tf.string)
-        self.upoints = tf.squeeze(
-            tf.strings.unicode_decode(
-                self.tokens, "UTF-8").to_tensor(shape=[None, 1])
-        )
+        self.upoints = tf.strings.unicode_decode(self.tokens, "UTF-8").to_tensor(shape=[None, 1])
 
     def extract(self, text: str) -> tf.Tensor:
         """
@@ -171,7 +167,7 @@ class CharFeaturizer(TextFeaturizer):
         with tf.name_scope("indices2upoints"):
             indices = self.normalize_indices(indices)
             upoints = tf.gather_nd(self.upoints, tf.expand_dims(indices, axis=-1))
-            return upoints
+            return tf.gather_nd(upoints, tf.where(tf.not_equal(upoints, 0)))
 
 
 class SubwordFeaturizer(TextFeaturizer):
@@ -236,6 +232,13 @@ class SubwordFeaturizer(TextFeaturizer):
         subwords = tds.features.text.SubwordTextEncoder.load_from_file(filename_prefix)
         return cls(decoder_config, subwords)
 
+    def save_to_file(self, filename: str = None):
+        if filename is not None:
+            filename_prefix = os.path.splitext(preprocess_paths(filename))[0]
+        else:
+            filename_prefix = self.decoder_config.vocabulary
+        return self.subwords.save_to_file(filename_prefix)
+
     def extract(self, text: str) -> tf.Tensor:
         """
         Convert string to a list of integers
@@ -259,18 +262,25 @@ class SubwordFeaturizer(TextFeaturizer):
         Returns:
             transcripts: tf.Tensor of dtype tf.string with dim [B]
         """
-        indices = self.normalize_indices(indices)
         with tf.device("/CPU:0"):  # string data is not supported on GPU
-            def decode(x):
-                if x[0] == self.blank: x = x[1:]
-                return self.subwords.decode(x)
-
-            text = tf.map_fn(
-                lambda x: tf.numpy_function(decode, inp=[x], Tout=tf.string),
-                indices,
-                fn_output_signature=tf.TensorSpec([], dtype=tf.string)
+            total = tf.shape(indices)[0]
+            batch = tf.constant(0, dtype=tf.int32)
+            transcripts = tf.TensorArray(
+                dtype=tf.string, size=total, dynamic_size=False, infer_shape=False,
+                clear_after_read=False, element_shape=tf.TensorShape([])
             )
-        return text
+
+            def cond(batch, total, transcripts): return tf.less(batch, total)
+
+            def body(batch, total, transcripts):
+                upoints = self.indices2upoints(indices[batch])
+                _transcript = tf.strings.unicode_encode(upoints, "UTF-8")
+                transcripts = transcripts.write(batch, _transcript)
+                return batch + 1, total, transcripts
+
+            _, _, transcripts = tf.while_loop(cond, body, loop_vars=[batch, total, transcripts])
+
+            return transcripts.stack()
 
     @tf.function(
         input_signature=[
@@ -289,6 +299,4 @@ class SubwordFeaturizer(TextFeaturizer):
         with tf.name_scope("indices2upoints"):
             indices = self.normalize_indices(indices)
             upoints = tf.gather_nd(self.upoints, tf.expand_dims(indices, axis=-1))
-            # upoints now has shape [None, max_subword_length]
-            shape = tf.shape(upoints)
-            return tf.reshape(upoints, [shape[0] * shape[1]])  # flatten
+            return tf.gather_nd(upoints, tf.where(tf.not_equal(upoints, 0)))
